%{

/*  VT1500 Device Driver for Linux, Version 1.0.10
 *  Copyright (C) 1995-2001  M. Gutschke
 *
 *  At the time of writing my e-mail address is:
 *	Internet: markus@gutschke.com
 *  My snail mail address is:
 *	Markus Gutschke
 *	3637 Fillmore Street #106
 *	San Francisco, CA 94123-1600
 *	USA
 *  If you like this software, I would appreciate if you sent me a postcard
 *  from your hometown. Under the terms of the GNU general public license
 *  you are free to include this program into (commercial) software
 *  distributions (e.g. putting it onto CD-ROM); nonetheless, I would really
 *  appreciate if you dropped me a short note (sending me a sample copy of
 *  your distribution would be even more appreciated!)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rc.h"
#include "tvcontrol.h"

#ifndef UNIX_PATH_MAX
#define UNIX_PATH_MAX 108
#endif


static char shortName[4],longName[21];
static int  frq,adj;

extern char *rctext;
extern int  rcReadOnly;

extern void rcerror(char *s);
extern int  rclex(void);

static inline int min(int a,int b) {return(a < b ? a : b);}
static inline int max(int a,int b) {return(a > b ? a : b);}

static int fromPercentage(int val,int def,int maxval)
{
  if (val < 0)
    return(min(maxval,max(0,(val*def-50)/100+def)));
  else if (val == 0)
    return(def);
  else
    return(min(maxval,max(0,(val*(maxval-def)+50)/100+def)));
}

char *strherror(int H_errno)
{
  static struct s {int i;char * s;} tab[] = {
    {HOST_NOT_FOUND,"No such host"},
    {TRY_AGAIN,"Name server does not respond; try again later"},
    {NO_RECOVERY,"Unexpected name server failure; cannot recover"},
    {NO_DATA,"Cannot obtain IP address for requested host"},
    {0,NULL}};
  static char buf[80];
  struct s *ptr = tab;
  while (ptr->s && ptr->i != H_errno) ptr++;
  if (!ptr->s) {sprintf(buf,"(h_errno: %d)",H_errno); return(buf); }
  else return(ptr->s);
}

%}

%union {
  int      iParm;
  char     *sParm;
}

%token <iParm> NUMBER FNUMBER
%token <sParm> NAME STRING
%token         READONLY SERVER EXTERNALMUTE MUTEONSWITCH PORT STATION MHZ
%token         RADIO PAL NTSC DISPLAY VGA TUNER SOURCE TV VCR CHANNEL MODE
%token         MUTE ON OFF BASS TREBLE VOLUME INTERNALVOLUME EXTERNALVOLUME
%token         BRIGHTNESS SATURATION CONTRAST HUE RED GREEN BLUE

%type  <iParm> smode
%type  <sParm> filename

%%

rc:	
	|	rc directive

directive: 	station
	|  	preference

station: 	STATION NUMBER ':' frequency MHZ shortname smode longname
					{ int index;
                                          if (adj < -99 || adj > 99 ||
                                              strcasecmp(shortName,
                                                         channelName(frq,
                                                                     &index)))
                                            rcerror("Invalid frequency");
                                          if ($2 < 0 || $2 >= 100)
                                            rcerror("Invalid station number");
                                          setChannelInfo(index,longName,adj,
                                                         $2,$7); }

frequency:	NUMBER			{ frq = ($1)*10; adj = 0; }
	|	NUMBER NUMBER		{ frq = ($1)*10; adj = ($2)*10;}
	|	NUMBER FNUMBER		{ frq = ($1)*10; adj = $2; }

shortname:	NUMBER			{ sprintf(shortName,"%d",(int)$1); }
	|	NAME			{ *shortName = '\000';
                                          strncat(shortName,$1,3);
					  free($1); }

smode:		RADIO			{ $$ = -1; }
	|	PAL			{ $$ =  1; }
	|	NTSC			{ $$ =  0; }

longname:				{ *longName = '\000'; }
	|	STRING			{ char *ptr;
                                          *longName = '\000';
                                          strncat(longName,$1+1,21); free($1);
                                          ptr = strrchr(longName,'\000');
                                          if (*--ptr == '"') *ptr = '\000'; }

preference:	readonly
	|	server
	|	externalmute
	|	muteonswitch
	|	port
	|	display
	|	source
	|	channel
	|	mode
	|	mute
	|	bass
	|	treble
	|	volume
	|	internalvolume
	|	externalvolume
	|	brightness
	|	saturation
	|	contrast
	|	hue
	|	red
	|	green
	|	blue

readonly:	READONLY		{ RCSettings.readonly = 1; }

server:		SERVER host ':' ipport
	|	SERVER filename		{ if (!RCSettings.serverOverride) {
					    RCSettings.s.server.sin_family =
					      AF_UNIX;
					    strncat(strcpy(RCSettings.s.
						    server_unix.sun_path,
						    ""),$2,UNIX_PATH_MAX); }
					  free($2); }

host:
	|	NAME			{ if (!RCSettings.serverOverride) {
					    struct hostent *he;
					    if (!(he = gethostbyname($1))) {
					      char buf[130];
					    hostnotfound:
					      sprintf(buf,"Cannot look up "
					    	"host: %s",strherror(h_errno));
					      rcerror(buf); break; }
					    RCSettings.s.server.sin_family =
					      he->h_addrtype;
					    memcpy(&RCSettings.s.server.
						   sin_addr,
					    	   he->h_addr,he->h_length); }
					  free($1); }
	|	NUMBER '.' NUMBER '.' NUMBER '.' NUMBER {
					  if (!RCSettings.serverOverride) {
					    struct hostent *he;
					    char buf[130];
					    if ((unsigned int)$1 > 255 ||
					    	(unsigned int)$3 > 255 ||
					    	(unsigned int)$5 > 255 ||
					    	(unsigned int)$7 > 255)
					      rcerror("Illegal host address");
					    sprintf(buf,"%d.%d.%d.%d",
					    	    $1,$3,$5,$7);
					    if (!(he = gethostbyname(buf)))
					      goto hostnotfound;
					    RCSettings.s.server.sin_family =
					      he->h_addrtype;
					    memcpy(&RCSettings.s.server.
						   sin_addr,
					    	   he->h_addr,he->h_length); }}
ipport:
	|	NUMBER			{ if (!RCSettings.serverOverride)
					    RCSettings.s.server.sin_port =
					      htons($1); }

filename:	'/' NAME		{ char *s = malloc(strlen($2)+2);
					  $$ = strcat(strcpy(s,"/"),$2);
					  free($2); }
	|	filename '/' NAME	{ char *s = realloc($1,strlen($1)+
							    strlen($3)+1);
					  $$ = strcat(strcat(s,"/"),$3);
					  free($3); }

externalmute:	EXTERNALMUTE		{ RCSettings.externalMute = 1; }

muteonswitch:	MUTEONSWITCH		{ RCSettings.muteOnSwitch = 1; }

port:		PORT NUMBER		{ if (!RCSettings.port)
                                            RCSettings.port = $2; }

display:	DISPLAY VGA		{ RCSettings.TVVGA = 0; }
	|	DISPLAY TUNER		{ RCSettings.TVVGA = 1; }

source:		SOURCE TV		{ RCSettings.TVVCR = 1; }
	|	SOURCE VCR		{ RCSettings.TVVCR = 0; }

channel:	CHANNEL NUMBER		{ RCSettings.channel = $2; }

mode:		MODE PAL		{ RCSettings.PALNTSC = 1; }
	|	MODE NTSC		{ RCSettings.PALNTSC = 0; }

mute:		MUTE ON			{ RCSettings.mute = 1; }
	|	MUTE OFF		{ RCSettings.mute = 0; }

bass:		BASS NUMBER		{ RCSettings.bass = fromPercentage
                                          ($2,0,MAXBASS); }
treble:		TREBLE NUMBER		{ RCSettings.treble = fromPercentage
                                          ($2,0,MAXTREBLE); }
volume:		VOLUME NUMBER		{ RCSettings.internalVolume =
					  RCSettings.externalVolume =
					    fromPercentage($2,0,MAXVOLUME); }
internalvolume:	INTERNALVOLUME NUMBER	{ RCSettings.internalVolume =
					    fromPercentage($2,0,MAXVOLUME); }
externalvolume:	EXTERNALVOLUME NUMBER	{ RCSettings.externalVolume =
					    fromPercentage($2,0,MAXVOLUME); }
brightness:	BRIGHTNESS NUMBER	{ RCSettings.brightness =fromPercentage
                                          ($2,DEFBRIGHTNESS,63); }
saturation:	SATURATION NUMBER	{ RCSettings.saturation =fromPercentage
                                          ($2,DEFSATURATION,63); }
contrast:	CONTRAST NUMBER		{ RCSettings.contrast = fromPercentage
                                          ($2,DEFCONTRAST,63); }
hue:		HUE NUMBER		{ RCSettings.hue = fromPercentage
                                          ($2,DEFHUE,255); }
red:		RED NUMBER		{ RCSettings.red = fromPercentage
                                          ($2,DEFRED,63); }
green:		GREEN NUMBER		{ RCSettings.green = fromPercentage
                                          ($2,DEFGREEN,63); }
blue:		BLUE NUMBER		{ RCSettings.blue = fromPercentage
                                          ($2,DEFBLUE,63); }
